import { WebSocketData, WebSocketClientConnectionProtocol, WebSocketClientConnection } from '@mswjs/interceptors/WebSocket';
import { Path } from '../utils/matching/matchRequestUrl.js';

declare const MSW_WEBSOCKET_CLIENTS_KEY = "msw:ws:clients";
type WebSocketBroadcastChannelMessage = {
    type: 'extraneous:send';
    payload: {
        clientId: string;
        data: WebSocketData;
    };
} | {
    type: 'extraneous:close';
    payload: {
        clientId: string;
        code?: number;
        reason?: string;
    };
};
/**
 * A manager responsible for accumulating WebSocket client
 * connections across different browser runtimes.
 */
declare class WebSocketClientManager {
    private channel;
    private url;
    private inMemoryClients;
    constructor(channel: BroadcastChannel, url: Path);
    /**
     * All active WebSocket client connections.
     */
    get clients(): Set<WebSocketClientConnectionProtocol>;
    private getSerializedClients;
    private addClient;
    /**
     * Adds the given `WebSocket` client connection to the set
     * of all connections. The given connection is always the complete
     * connection object because `addConnection()` is called only
     * for the opened connections in the same runtime.
     */
    addConnection(client: WebSocketClientConnection): void;
}
/**
 * A wrapper class to operate with WebSocket client connections
 * from other runtimes. This class maintains 1-1 public API
 * compatibility to the `WebSocketClientConnection` but relies
 * on the given `BroadcastChannel` to communicate instructions
 * with the client connections from other runtimes.
 */
declare class WebSocketRemoteClientConnection implements WebSocketClientConnectionProtocol {
    readonly id: string;
    readonly url: URL;
    private channel;
    constructor(id: string, url: URL, channel: BroadcastChannel);
    send(data: WebSocketData): void;
    close(code?: number | undefined, reason?: string | undefined): void;
}

export { MSW_WEBSOCKET_CLIENTS_KEY, type WebSocketBroadcastChannelMessage, WebSocketClientManager, WebSocketRemoteClientConnection };
