"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var WebSocketClientManager_exports = {};
__export(WebSocketClientManager_exports, {
  MSW_WEBSOCKET_CLIENTS_KEY: () => MSW_WEBSOCKET_CLIENTS_KEY,
  WebSocketClientManager: () => WebSocketClientManager,
  WebSocketRemoteClientConnection: () => WebSocketRemoteClientConnection
});
module.exports = __toCommonJS(WebSocketClientManager_exports);
var import_outvariant = require("outvariant");
var import_matchRequestUrl = require("../utils/matching/matchRequestUrl.js");
const MSW_WEBSOCKET_CLIENTS_KEY = "msw:ws:clients";
class WebSocketClientManager {
  constructor(channel, url) {
    this.channel = channel;
    this.url = url;
    this.inMemoryClients = /* @__PURE__ */ new Set();
    if (typeof localStorage !== "undefined") {
      localStorage.removeItem = new Proxy(localStorage.removeItem, {
        apply: (target, thisArg, args) => {
          const [key] = args;
          if (key === MSW_WEBSOCKET_CLIENTS_KEY) {
            this.inMemoryClients.clear();
          }
          return Reflect.apply(target, thisArg, args);
        }
      });
    }
  }
  inMemoryClients;
  /**
   * All active WebSocket client connections.
   */
  get clients() {
    if (typeof localStorage !== "undefined") {
      const inMemoryClients = Array.from(this.inMemoryClients);
      console.log("get clients()", inMemoryClients, this.getSerializedClients());
      return new Set(
        inMemoryClients.concat(
          this.getSerializedClients().filter((serializedClient) => {
            if (inMemoryClients.every(
              (client) => client.id !== serializedClient.clientId
            )) {
              return serializedClient;
            }
          }).map((serializedClient) => {
            return new WebSocketRemoteClientConnection(
              serializedClient.clientId,
              new URL(serializedClient.url),
              this.channel
            );
          })
        )
      );
    }
    return this.inMemoryClients;
  }
  getSerializedClients() {
    (0, import_outvariant.invariant)(
      typeof localStorage !== "undefined",
      "Failed to call WebSocketClientManager#getSerializedClients() in a non-browser environment. This is likely a bug in MSW. Please, report it on GitHub: https://github.com/mswjs/msw"
    );
    const clientsJson = localStorage.getItem(MSW_WEBSOCKET_CLIENTS_KEY);
    if (!clientsJson) {
      return [];
    }
    const allClients = JSON.parse(clientsJson);
    const matchingClients = allClients.filter((client) => {
      return (0, import_matchRequestUrl.matchRequestUrl)(new URL(client.url), this.url).matches;
    });
    return matchingClients;
  }
  addClient(client) {
    this.inMemoryClients.add(client);
    if (typeof localStorage !== "undefined") {
      const serializedClients = this.getSerializedClients();
      const nextSerializedClients = serializedClients.concat({
        clientId: client.id,
        url: client.url.href
      });
      localStorage.setItem(
        MSW_WEBSOCKET_CLIENTS_KEY,
        JSON.stringify(nextSerializedClients)
      );
    }
  }
  /**
   * Adds the given `WebSocket` client connection to the set
   * of all connections. The given connection is always the complete
   * connection object because `addConnection()` is called only
   * for the opened connections in the same runtime.
   */
  addConnection(client) {
    this.addClient(client);
    const handleExtraneousMessage = (message) => {
      const { type, payload } = message.data;
      if (typeof payload === "object" && "clientId" in payload && payload.clientId !== client.id) {
        return;
      }
      switch (type) {
        case "extraneous:send": {
          client.send(payload.data);
          break;
        }
        case "extraneous:close": {
          client.close(payload.code, payload.reason);
          break;
        }
      }
    };
    const abortController = new AbortController();
    this.channel.addEventListener("message", handleExtraneousMessage, {
      signal: abortController.signal
    });
    client.addEventListener("close", () => abortController.abort(), {
      once: true
    });
  }
}
class WebSocketRemoteClientConnection {
  constructor(id, url, channel) {
    this.id = id;
    this.url = url;
    this.channel = channel;
  }
  send(data) {
    this.channel.postMessage({
      type: "extraneous:send",
      payload: {
        clientId: this.id,
        data
      }
    });
  }
  close(code, reason) {
    this.channel.postMessage({
      type: "extraneous:close",
      payload: {
        clientId: this.id,
        code,
        reason
      }
    });
  }
}
//# sourceMappingURL=WebSocketClientManager.js.map