import { WebSocketHandler, kDispatchEvent } from '../handlers/WebSocketHandler.mjs';
import { webSocketInterceptor } from './webSocketInterceptor.mjs';
function handleWebSocketEvent(options) {
  webSocketInterceptor.on("connection", (connection) => {
    const handlers = options.getHandlers();
    const connectionEvent = new MessageEvent("connection", {
      data: connection
    });
    const matchingHandlers = handlers.filter(
      (handler) => {
        if (handler instanceof WebSocketHandler) {
          return handler.predicate({
            event: connectionEvent,
            parsedResult: handler.parse({
              event: connectionEvent
            })
          });
        }
        return false;
      }
    );
    if (matchingHandlers.length > 0) {
      options?.onMockedConnection(connection);
      for (const handler of matchingHandlers) {
        handler[kDispatchEvent](connectionEvent);
      }
    } else {
      options?.onPassthroughConnection(connection);
      connection.server.connect();
    }
  });
}
export {
  handleWebSocketEvent
};
//# sourceMappingURL=handleWebSocketEvent.mjs.map