import { WebSocketConnectionData, WebSocketClientConnection, WebSocketData } from '@mswjs/interceptors/WebSocket';

declare function attachWebSocketLogger(connection: WebSocketConnectionData): void;
/**
 * Prints the WebSocket connection.
 * This is meant to be logged by every WebSocket handler
 * that intercepted this connection. This helps you see
 * what handlers observe this connection.
 */
declare function logConnectionOpen(client: WebSocketClientConnection): void;
/**
 * Prints the outgoing client message.
 */
declare function logOutgoingClientMessage(event: MessageEvent<WebSocketData>): Promise<void>;
/**
 * Prints the outgoing client message initiated
 * by `server.send()` in the event handler.
 */
declare function logOutgoingMockedClientMessage(event: MessageEvent<WebSocketData>): Promise<void>;
/**
 * Prings the message received by the WebSocket client.
 * This is fired when the "message" event is dispatched
 * on the actual WebSocket client instance, and translates to
 * the client receiving a message from the server.
 */
declare function logIncomingClientMessage(event: MessageEvent<WebSocketData>): Promise<void>;
/**
 * Prints the outgoing client message initiated
 * by `client.send()` in the event handler.
 */
declare function logIncomingMockedClientMessage(event: MessageEvent<WebSocketData>): Promise<void>;
declare function logIncomingServerMessage(event: MessageEvent<WebSocketData>): Promise<void>;

export { attachWebSocketLogger, logConnectionOpen, logIncomingClientMessage, logIncomingMockedClientMessage, logIncomingServerMessage, logOutgoingClientMessage, logOutgoingMockedClientMessage };
